package app;

import characters.Player;
import characters.Ninja;
import characters.robot.Robot;
import com.jme3.bullet.BulletAppState;
import com.jme3.bullet.control.RigidBodyControl;
import com.jme3.input.KeyInput;
import com.jme3.input.controls.ActionListener;
import com.jme3.input.controls.KeyTrigger;
import com.jme3.light.AmbientLight;
import com.jme3.light.DirectionalLight;
import com.jme3.light.PointLight;
import com.jme3.light.SpotLight;
import com.jme3.math.ColorRGBA;
import com.jme3.math.FastMath;
import com.jme3.math.Vector2f;
import com.jme3.math.Vector3f;
import com.jme3.renderer.queue.RenderQueue;
import com.jme3.scene.Node;
import com.jme3.scene.Spatial;
import models.DirtyAreaLocations;
import models.Tree;

public class Game extends App {

    private BulletAppState bulletAppState;
    private Node terrain;
    //player
    private Player player;
    //Ninja
    private Ninja ninja;
    private Ninja ninja1;
    private Ninja ninja2;
    //Robot
    private Robot robot;
    //robot cleeaning places
    private Vector3f[] cleanArea = {new Vector3f(20f, 0f, 20f),
        new Vector3f(-20f, 0f, 20f)};
    private Vector3f treePositions[] = new Vector3f[]{new Vector3f(-15.5f, 0f, -3f), new Vector3f(-46f, 0, 32), new Vector3f(6.8f, 0, 40f), new Vector3f(69.5f, 0, -2.5f), new Vector3f(40f, 0, -60f), new Vector3f(-46f, 0, -60f)};

    public void loadLevel() {
        terrain = (Node) assetManager.loadModel("Scenes/MonkeyZone.j3o");
        RigidBodyControl physic = new RigidBodyControl(0);
        terrain.addControl(physic);
        physic.setFriction(1f);
        physic.setRestitution(1f);
        rootNode.attachChild(terrain);
        //add light to scene
        setSunShine();
        //load Ninja
        ninja = new Ninja(rootNode, bulletAppState, assetManager, "Models/Ninja/Ninja.mesh.xml", "NonPlayerFalse");
        ninja1 = new Ninja(rootNode, bulletAppState, assetManager, "Models/Ninja/Ninja.mesh.xml", "NonPlayerFalse");
        ninja2 = new Ninja(rootNode, bulletAppState, assetManager, "Models/Ninja/Ninja.mesh.xml", "NonPlayerFalse");
        //load Robot
        robot = new Robot(rootNode, bulletAppState, assetManager, "Models/Oto/Oto.mesh.xml", "Robot");
        //load Player
        player = new Player(rootNode, bulletAppState, assetManager, inputManager, "Models/Sinbad/Sinbad.mesh.xml", "Player");

        for (int i = 0; i < treePositions.length; i++) {
            new Tree(rootNode, bulletAppState, assetManager, treePositions[i]);
        }

        bulletAppState.getPhysicsSpace().add(terrain);
    }
    private ActionListener actionListener = new ActionListener() {
        public void onAction(String binding, boolean keyPressed, float tpf) {
            if (binding.equals("Escape")) {
                System.exit(0);
            }
        }
    };

    @Override
    public void appUpdate(float tpf) {
        player.update(tpf);
        ninja.update(tpf);
        ninja1.update(tpf);
        ninja2.update(tpf);
        robot.update(tpf);

        setupCamera();
    }

    @Override
    public void initialize() {
        super.initialize();
        guiFont = loadGuiFont();
        guiNode.setQueueBucket(RenderQueue.Bucket.Gui);
        guiNode.setCullHint(Spatial.CullHint.Never);
        viewPort.attachScene(rootNode);
        guiViewPort.attachScene(guiNode);
        bulletAppState = new BulletAppState();
        bulletAppState.setThreadingType(BulletAppState.ThreadingType.PARALLEL);
        stateManager.attach(bulletAppState);
        appInit();
    }

    public void appInit() {
        initKeys();
        loadLevel();
        setupCamera();
    }

    private void initKeys() {
        inputManager.addMapping("Escape", new KeyTrigger(KeyInput.KEY_ESCAPE));
        inputManager.addListener(actionListener, "Escape");
    }

    private void setupCamera() {
        //camera on player
        cam.setLocation(player.getNodeForMovement().getLocalTranslation().
                add(player.getSpatialForRotation().getLocalRotation().getRotationColumn(2).
                negate().multLocal(30f)).add(new Vector3f(0f, 15f, 0f)));   //
        cam.lookAt(player.getNodeForMovement().getLocalTranslation().
                add(new Vector3f(0f, 10f, 0f)), Vector3f.UNIT_Y);
    }

    private void setSunShine() {
        //add sun shine to simulation
        DirectionalLight dl = new DirectionalLight();
        dl.setColor(ColorRGBA.White);
        dl.setDirection(new Vector3f(2.8f, -2.8f, -2.8f).normalizeLocal());
        rootNode.addLight(dl);
        //add sun shine to other side for visibility of model 
        dl.setColor(ColorRGBA.LightGray);
        dl.setDirection(new Vector3f(2.8f, -2.8f, +2.8f).normalizeLocal());
        rootNode.addLight(dl);
    }
}
