package app;

import com.jme3.system.AppSettings;
import com.jme3.system.JmeCanvasContext;
import com.jme3.util.JmeFormatter;
import java.awt.Canvas;
import java.awt.Dimension;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.logging.ConsoleHandler;
import java.util.logging.Handler;
import java.util.logging.Logger;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;

public class Main {

    private static JmeCanvasContext context;
    private static Canvas canvas;
    private static Game app;
    private static JFrame window;
    private static Layout layout;
    private static final String appClass = "app.Game";
    private static Dimension windowSize = new Dimension(1024, 680);

    public static void main(String[] args) {

        JmeFormatter formatter = new JmeFormatter();

        Handler consoleHandler = new ConsoleHandler();
        consoleHandler.setFormatter(formatter);

        createCanvas(appClass);

        try {
            Thread.sleep(500);
        } catch (InterruptedException ex) {
        }

        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                JPopupMenu.setDefaultLightWeightPopupEnabled(false);

                createFrame();

                window.pack();
                startApp();
                window.setLocationRelativeTo(null);
                window.setVisible(true);
            }
        });


    }

    private static void createFrame() {
        window = new JFrame("CleanBot game with executable prototype controller");

        window.setMaximumSize(canvas.getMaximumSize());
        window.setMinimumSize(canvas.getMinimumSize());
        window.setSize(windowSize);
        window.setResizable(false);

        window.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        window.addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosed(WindowEvent e) {
                app.stop();
            }
        });

        layout = new Layout(app);
        JPanel sceneContainer = layout.getSceneContainer();
        sceneContainer.add(canvas);
        layout.setSize(windowSize);
        window.add(layout);
    }

    public static void createCanvas(String appClass) {
        AppSettings settings = new AppSettings(true);
        settings.setWidth(windowSize.width);
        settings.setHeight(windowSize.height);
        settings.setAudioRenderer(null);
        settings.setSamples(2);

        app = new Game();

        app.setPauseOnLostFocus(false);
        app.setSettings(settings);
        app.createCanvas();
        app.startCanvas();

        context = (JmeCanvasContext) app.getContext();
        canvas = context.getCanvas();
        canvas.setSize(settings.getWidth(), settings.getHeight());
    }

    public static void startApp() {
        // Setup console
        JmeFormatter formatter = new JmeFormatter();
        Handler consoleHandler = new ConsoleHandler();
        consoleHandler.setFormatter(formatter);
        Logger.getLogger("").removeHandler(Logger.getLogger("").getHandlers()[0]);
        Logger.getLogger("").addHandler(consoleHandler);

        app.startCanvas();
    }

    public static Layout getLayout() {
        return layout;
    }

    public static JFrame getWindow() {
        return window;
    }
}
