/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package characters;

import app.Game;
import com.jme3.animation.AnimChannel;
import com.jme3.animation.AnimControl;
import com.jme3.animation.LoopMode;
import com.jme3.asset.AssetManager;
import com.jme3.bounding.BoundingBox;
import com.jme3.bullet.BulletAppState;
import com.jme3.bullet.control.BetterCharacterControl;
import com.jme3.math.Vector3f;
import com.jme3.scene.Node;
import com.jme3.scene.Spatial;
import java.awt.Component;
import java.util.Random;
import javax.swing.JOptionPane;

/**
 *
 * @author peter
 */
public class Ninja extends AbstractCharacter implements OtherCharacter {

    private final String ANIM_CHANNEL_NAME = "ninjaDefault";
    private float rotationAngleForNinja;
    private NinjaThread threadGeneratePoint;
    private Vector3f walkDirectionNinja = new Vector3f();

    public Ninja(Node rootNode, BulletAppState bulletAppState, AssetManager assetManager, String path, String name) {
        super();
        Spatial playerSpatial = assetManager.loadModel(path);
        
//        playerSpatial.setName(name);
        this.setName(name);
        setSpatialForRotation(playerSpatial);// You can set the model directly to the player. (We just wanted to explicitly show that it's a spatial.)
        getSpatialForRotation().setLocalScale(0.05f); // optionally adjust scale of model
        setNodeForMovement(new Node()); // You can create a new node to wrap your player to adjust the location. (This allows you to solve issues with character sinking into floor, etc.)
        getNodeForMovement().attachChild(getSpatialForRotation()); // add it to the wrapper

        // setup animation:
        setAnimControl(getSpatialForRotation().getControl(AnimControl.class));
        setAnimChannel(ANIM_CHANNEL_NAME, getAnimControl().createChannel());
        getAnimChannel(ANIM_CHANNEL_NAME).setAnim("Idle2");

        setCharacterControl(new BetterCharacterControl(1.5f, 6f, 1f)); // construct character. (If your character bounces, try increasing height and weight.)
        getNodeForMovement().addControl(getCharacterControl()); // attach to wrapper
        // add to physics state
        bulletAppState.getPhysicsSpace().add(getCharacterControl());
        bulletAppState.getPhysicsSpace().addAll(getNodeForMovement());
        rootNode.attachChild(getNodeForMovement()); // add wrapper to root
        ((Node) rootNode.getChild("New Scene")).attachChild(getNodeForMovement());

        bulletAppState.getPhysicsSpace().add(getCharacterControl());
        // set basic physical properties:
        getCharacterControl().setJumpForce(new Vector3f(0, 2f, 0));
        getCharacterControl().setGravity(new Vector3f(0, -50f, 0));

        rootNode.attachChild(this);

        threadGeneratePoint = new NinjaThread(this);
        threadGeneratePoint.start();

        //print to terminal all possible animation
        System.out.println("Ninja" + this.getAnimControl().getAnimationNames());
    }

    @Override
    public void update(float tpf) {
        if (getIsAlive()) {
            walkHandler();
            animationHandler(tpf);
        } else {
            walkDirectionNinja.set(0, 0, 0);
            if (!"Death1".equals(getAnimChannel(ANIM_CHANNEL_NAME).getAnimationName())) {
                getAnimChannel(ANIM_CHANNEL_NAME).setAnim("Death1");
                getAnimChannel(ANIM_CHANNEL_NAME).setLoopMode(LoopMode.DontLoop);
                threadGeneratePoint.interrupt();
            }
            getCharacterControl().setWalkDirection(walkDirectionNinja);
        }
    }

    @Override
    public void animationHandler(float tpf) {
        if (walkDirectionNinja.lengthSquared() == 0) { //Use lengthSquared() (No need for an extra sqrt())
            if (!"Idle2".equals(getAnimChannel(ANIM_CHANNEL_NAME).getAnimationName())) {
                getAnimChannel(ANIM_CHANNEL_NAME).setAnim("Idle2");
            }
        } else {
            if (!"Walk".equals(getAnimChannel(ANIM_CHANNEL_NAME).getAnimationName())) {
                getAnimChannel(ANIM_CHANNEL_NAME).setAnim("Walk");
            }
        }
    }

    @Override
    public void walkHandler() {
        walkDirectionNinja.set(0, 0, 0);
        walkDirectionNinja.addLocal(getSpatialForRotation().getLocalRotation().getRotationColumn(2).multLocal(7f).negate());
        getSpatialForRotation().rotate(0, rotationAngleForNinja, 0);
        getCharacterControl().setWalkDirection(walkDirectionNinja);
    }

    private class NinjaThread extends Thread {

        private Ninja ninja;

        private NinjaThread(Ninja aThis) {
            this.ninja = aThis;
        }

        @Override
        public void run() {
            Random gen = new Random();

            while (true) {
                try {
                    if (gen.nextBoolean()) {
                        ninja.rotationAngleForNinja = (gen.nextFloat() / 30) * (gen.nextBoolean() ? 1 : -1);
                    } else {
                        ninja.rotationAngleForNinja = 0f;
                    }

                    Thread.sleep((2 + gen.nextInt(2)) * 1000);
                } catch (Exception ex) {
                    System.out.println(ex);
                }
            }
        }
    }
}
