/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package characters;

import com.jme3.animation.AnimChannel;
import com.jme3.animation.AnimControl;
import com.jme3.animation.LoopMode;
import com.jme3.asset.AssetManager;
import com.jme3.bullet.BulletAppState;
import com.jme3.bullet.control.BetterCharacterControl;
import com.jme3.input.InputManager;
import com.jme3.input.KeyInput;
import com.jme3.input.controls.ActionListener;
import com.jme3.input.controls.KeyTrigger;
import com.jme3.math.Vector3f;
import com.jme3.scene.Node;
import com.jme3.scene.Spatial;
import java.awt.Component;
import javax.swing.JOptionPane;

/**
 *
 * @author peter
 */
public class Player extends AbstractCharacter implements PlayerCharacter {

    private boolean leftward = false;
    private boolean rightward = false;
    private boolean forward = false;
    private boolean backward = false;
    private boolean jump = false;
    private Vector3f walkDirection = new Vector3f();
    private AnimChannel base;
    private AnimChannel top;
    private float airTime = 0;
    private int delayAfterDead = 100;
    private final InputManager inputManager;

    public Player(Node rootNode, BulletAppState bulletAppState, AssetManager assetManager, InputManager inputManager, String path, String name) {
        super();
        this.inputManager = inputManager;
        initKeys();

        Spatial sinbadSpatial = assetManager.loadModel(path);

//        sinbadSpatial.setName(name);
        this.setName(name);
        setSpatialForRotation((Node) sinbadSpatial); // You can set the model directly to the player. (We just wanted to explicitly show that it's a spatial.)
        getSpatialForRotation().setLocalTranslation(0f, 5f, 0f);
        setNodeForMovement(new Node()); // You can create a new node to wrap your player to adjust the location. (This allows you to solve issues with character sinking into floor, etc.)
        getNodeForMovement().attachChild(getSpatialForRotation()); // add it to the wrapper
        getNodeForMovement().move(0, 10f, 0); // adjust position to ensure collisions occur correctly.

        // setup animation:
        setAnimControl(getSpatialForRotation().getControl(AnimControl.class));

        setAnimChannel("Base", getAnimControl().createChannel());
        getAnimChannel("Base").setAnim("IdleBase");

        setAnimChannel("Top", getAnimControl().createChannel());
        getAnimChannel("Top").setAnim("IdleTop");

        setCharacterControl(new BetterCharacterControl(1.5f, 6f, 1f)); // construct character. (If your character bounces, try increasing height and weight.)
        getNodeForMovement().addControl(getCharacterControl()); // attach to wrapper
        // add to physics state
        bulletAppState.getPhysicsSpace().add(getCharacterControl());
        bulletAppState.getPhysicsSpace().addAll(getNodeForMovement());
        rootNode.attachChild(getNodeForMovement()); // add wrapper to root
        ((Node) rootNode.getChild("New Scene")).attachChild(getNodeForMovement());

        bulletAppState.getPhysicsSpace().add(getCharacterControl());

        // set basic physical properties:
        getCharacterControl().setJumpForce(new Vector3f(0, 10f, 0));
        getCharacterControl().setGravity(new Vector3f(0, -50f, 0));
        getCharacterControl().warp(new Vector3f(15f, 0f, 0)); // warp character into landscape at particular location
        base = getAnimChannel("Base");
        top = getAnimChannel("Top");
        rootNode.attachChild(this);
        //print to terminal all possible animation
        System.out.println(this.getAnimControl().getAnimationNames());
    }
    private ActionListener actionListener = new ActionListener() {
        public void onAction(String binding, boolean keyPressed, float tpf) {
            if (binding.equals("CharLeft")) {
                leftward = keyPressed;
            } else if (binding.equals("CharRight")) {
                rightward = keyPressed;
            } else if (binding.equals("CharForward")) {
                forward = keyPressed;
            } else if (binding.equals("CharBackward")) {
                backward = keyPressed;
            } else if (binding.equals("Jump")) {
                jump = keyPressed;
            }
        }
    };

    @Override
    public void update(float tpf) {
        if (getIsAlive()) {
            walkDirection.set(0, 0, 0);
            walkHandler();
            animationHandler(tpf);
            getCharacterControl().setWalkDirection(walkDirection);
        } else {
            walkDirection.set(0, 0, 0);
            if (!"HandsClosed".equals(getAnimChannel("Base").getAnimationName())) {
                base.setAnim("HandsClosed");
                setAnimChannel("Base", base);
                top.setAnim("StandUpFront");
                top.setLoopMode(LoopMode.DontLoop);
                setAnimChannel("Top", top);
            }
            if (delayAfterDead == 0) {
                if (JOptionPane.showConfirmDialog((Component) null, "You managed to kill your characted.\nWould you like to quit the game? ", "Congratulations!", JOptionPane.OK_CANCEL_OPTION) == 0) {
                    System.exit(0);
                }
                delayAfterDead--;
            } else if (delayAfterDead > 0) {
                delayAfterDead--;
            }
            getCharacterControl().setWalkDirection(walkDirection);
        }
    }

    @Override
    public void animationHandler(float tpf) {
        if (!getCharacterControl().isOnGround()) { // use !character.isOnGround() if the character is a BetterCharacterControl type.
            airTime += tpf;
        } else {
            airTime = 0;
        }
        if (walkDirection.lengthSquared() == 0) { //Use lengthSquared() (No need for an extra sqrt())
            if (!"IdleBase".equals(getAnimChannel("Base").getAnimationName())) {
                base.setAnim("IdleBase");
                setAnimChannel("Base", base);
                top.setAnim("IdleTop");
                setAnimChannel("Top", top);
            }
        } else {
            if (!getCharacterControl().isOnGround()) {
                if (!"JumpLoop".equals(getAnimChannel("Base").getAnimationName())) {
                    base.setAnim("JumpLoop");
                    setAnimChannel("Base", base);
                    top.setAnim("JumpLoop");
                    setAnimChannel("Top", top);
                }
            } else if (!"RunBase".equals(getAnimChannel("Base").getAnimationName())) {
                base.setAnim("RunBase");
                setAnimChannel("Base", base);
                top.setAnim("RunTop");
                setAnimChannel("Top", top);
            }
        }

    }

    @Override
    public void walkHandler() {
        if (jump) {
            getCharacterControl().jump();
        }
        if (leftward) {
            getSpatialForRotation().rotate(0, 0.025f, 0);
        }
        if (rightward) {
            getSpatialForRotation().rotate(0, -0.025f, 0);
        }
        if (forward) {
            walkDirection.addLocal(
                    getSpatialForRotation().getLocalRotation().
                    getRotationColumn(2).multLocal(15f));
        }
        if (backward) {
            walkDirection.addLocal(
                    getSpatialForRotation().getLocalRotation().
                    getRotationColumn(2).negate().multLocal(15f));
        }
        getCharacterControl().setWalkDirection(walkDirection);
    }

    public void initKeys() {

        inputManager.addMapping("Jump", new KeyTrigger(KeyInput.KEY_SPACE)); // trigger 1: spacebar
        inputManager.addMapping("CharLeft", new KeyTrigger(KeyInput.KEY_LEFT));
        inputManager.addMapping("CharRight", new KeyTrigger(KeyInput.KEY_RIGHT));
        inputManager.addMapping("CharForward", new KeyTrigger(KeyInput.KEY_UP));
        inputManager.addMapping("CharBackward", new KeyTrigger(KeyInput.KEY_DOWN));

        inputManager.addMapping("CharLeft", new KeyTrigger(KeyInput.KEY_A));
        inputManager.addMapping("CharRight", new KeyTrigger(KeyInput.KEY_D));
        inputManager.addMapping("CharForward", new KeyTrigger(KeyInput.KEY_W));
        inputManager.addMapping("CharBackward", new KeyTrigger(KeyInput.KEY_S));

        inputManager.addListener(actionListener,
                "Jump", "CharLeft", "CharRight", "CharForward", "CharBackward");
    }
}