/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package characters.robot;

import app.Measure;
import characters.AbstractCharacter;
import characters.OtherCharacter;
import characters.PlayerCharacter;
import com.jme3.animation.AnimChannel;
import com.jme3.animation.AnimControl;
import com.jme3.asset.AssetManager;
import com.jme3.bullet.BulletAppState;
import com.jme3.bullet.control.BetterCharacterControl;
import com.jme3.light.PointLight;
import com.jme3.math.ColorRGBA;
import com.jme3.math.Vector3f;
import com.jme3.scene.Node;
import com.jme3.scene.Spatial;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import models.DirtyAreaLocations;
import models.DirtyAreaLocations.DirtyArea;

/**
 *
 * @author peter
 */
public class Robot extends AbstractCharacter implements RobotCharacter {

    private CBotController safetyController;
    private DirtyAreaLocations dirtyAreaLocations;
    private Node rootNode;
    private String ANIM_CHANNEL_NAME = "RobotDefault";
    private AnimChannel anim;
    private PointLight lightForCleaning;
    private Boolean lightOn = false;
    private Boolean ignoreOtherCharacters = true;
    private float time = 0;
    private final float updateFrequency = (1 / 30); //hodnota 1 by mala byt 1 sec, cize 1/30 je tridsiatina sekundy
    private Vector3f walkDirectionRobot = new Vector3f();

    public Robot(Node rootNode, BulletAppState bulletAppState, AssetManager assetManager, String path, String name) {
        loadRobot(assetManager, path, name, bulletAppState, rootNode);
        this.rootNode = rootNode;
        dirtyAreaLocations = new DirtyAreaLocations(rootNode);
        safetyController = new CBotController(500, 300, 200, 200);  //parametre v cm
    }

    @Override
    public void update(float tpf) {
        animationHandler(tpf);
        //vykonanie riadiaceho programu v konštantnej frekvencii
        time += tpf;
        if (time > updateFrequency) {
            execute();
            time = 0;
        }
    }

    public void animationHandler(float tpf) {
        if (walkDirectionRobot.lengthSquared() == 0) { //Use lengthSquared() (No need for an extra sqrt())
            if (!"stand".equals(getAnimChannel(ANIM_CHANNEL_NAME).getAnimationName())) {
                getAnimChannel(ANIM_CHANNEL_NAME).setAnim("stand");
            }
        } else {
            if (!"Walk".equals(getAnimChannel(ANIM_CHANNEL_NAME).getAnimationName())) {
                getAnimChannel(ANIM_CHANNEL_NAME).setAnim("Walk");
            }
        }
        lightForCleaning.setPosition(getBotPosition().add(0f, 20f, 0f));
    }

    @Override
    public void walkHandler() {
        //namiesto tejto metódy sa používa metóda go(Vector),
        //ktorá je rozšírená o smer, ktorým sa má robot pohnúť
    }

    public void go(Vector3f direction) {
        walkDirectionRobot.set(0f, 0f, 0f);
        walkDirectionRobot.addLocal(direction);
        if (walkDirectionRobot.length() != 0) {
            getCharacterControl().setViewDirection(walkDirectionRobot);
        }
        getCharacterControl().setWalkDirection(walkDirectionRobot);
    }

    public void turnLightOn() {
        if (!lightOn) {
            for (int i = 0; i < 10; i++) {
                getNodeForMovement().addLight(lightForCleaning);
            }
            lightOn = true;
        }
    }

    public void turnLightOff() {
        if (lightOn) {
            for (int i = 0; i < 10; i++) {
                getNodeForMovement().removeLight(lightForCleaning);
            }
            lightOn = false;
        }


    }

    public void clean() {
        Iterator iterator = dirtyAreaLocations.getDirtyAreas().iterator();
        DirtyArea dirtyArea;
        while (iterator.hasNext()) {
            dirtyArea = (DirtyArea) iterator.next();
            if ((getBotPosition().distance(dirtyArea.getPosition())) < 2f) {
                dirtyArea.cleanOneUnit();
            }
        }
    }

    public Vector3f getNextPosition() {
        Iterator iterator = dirtyAreaLocations.getDirtyAreas().iterator();

        DirtyArea dirtyArea;
        while (iterator.hasNext()) {
            dirtyArea = (DirtyArea) iterator.next();
            if (dirtyArea.getDirtiness() > 0) {
                return dirtyArea.getPosition();
            }
        }
        return new Vector3f(0, 0, 0);
    }

    public List<Node> getPlayerPositions() {
        List<Spatial> children = rootNode.getChildren();
        List<Node> nodes = new ArrayList<Node>();

        for (Spatial spatial : children) {
            if (spatial instanceof PlayerCharacter) {
                nodes.add((Node) spatial);
            }
            if (!ignoreOtherCharacters && spatial instanceof OtherCharacter) {
                nodes.add((Node) spatial);
            }
        }

        return nodes;
    }

    public List<Node> getAllPlayerPositions() {
        List<Spatial> children = rootNode.getChildren();
        List<Node> nodes = new ArrayList<Node>();

        for (Spatial spatial : children) {
            if (spatial instanceof AbstractCharacter && !(spatial instanceof RobotCharacter)) {
                nodes.add((Node) spatial);
            }
        }

        return nodes;
    }

    public Vector3f getBotPosition() {
        return this.getNodeForMovement().getLocalTranslation();
    }

    private void loadRobot(AssetManager assetManager, String path, String name, BulletAppState bulletAppState, Node rootNode) {
        Spatial playerSpatial = assetManager.loadModel(path);
        this.setName(name);
        setSpatialForRotation(playerSpatial); // You can set the model directly to the player. (We just wanted to explicitly show that it's a spatial.)
        getSpatialForRotation().setLocalTranslation(0f, 5f, 0f);
        setNodeForMovement(new Node()); // You can create a new node to wrap your player to adjust the location. (This allows you to solve issues with character sinking into floor, etc.)
        getNodeForMovement().attachChild(getSpatialForRotation()); // add it to the wrapper

        // setup animation:
        setAnimControl(getSpatialForRotation().getControl(AnimControl.class));
        setAnimChannel(ANIM_CHANNEL_NAME, getAnimControl().createChannel());
        getAnimChannel(ANIM_CHANNEL_NAME).setAnim("stand");

        setCharacterControl(new BetterCharacterControl(1.5f, 6f, 1f)); // construct character. (If your character bounces, try increasing height and weight.)
        getNodeForMovement().addControl(getCharacterControl()); // attach to wrapper

        // add to physics state
        bulletAppState.getPhysicsSpace().add(getCharacterControl());
        bulletAppState.getPhysicsSpace().addAll(getNodeForMovement());
        rootNode.attachChild(getNodeForMovement()); // add wrapper to root
        ((Node) rootNode.getChild("New Scene")).attachChild(getNodeForMovement());

        bulletAppState.getPhysicsSpace().add(getCharacterControl());

        // set basic physical properties:
        getCharacterControl().setJumpForce(new Vector3f(0, 2f, 0));
        getCharacterControl().setGravity(new Vector3f(0, -50f, 0));
        getCharacterControl().warp(new Vector3f(7f, 0f, 0)); // warp character into landscape at particular location

        rootNode.attachChild(this);

        anim = getAnimChannel(ANIM_CHANNEL_NAME);
        //light while robot isCleaning
        lightForCleaning = new PointLight();
        lightForCleaning.setPosition(getBotPosition().multLocal(15f));
        lightForCleaning.setColor(ColorRGBA.Yellow);
        lightForCleaning.setRadius(20f);
        lightForCleaning.setName("lightForCleaning");
        rootNode.addLight(lightForCleaning);
        //print to terminal all possible animation
        System.out.println(getAnimControl().getAnimationNames());
    }

    @Override
    public Boolean getIgnoreOtherCharacters() {
        return ignoreOtherCharacters;
    }

    @Override
    public void setIgnoreOtherCharacters(Boolean ignoreOtherCharacters) {
        this.ignoreOtherCharacters = ignoreOtherCharacters;
    }

    private void execute() {

        //priprava dat pre riadiaci program
        Vector3f nPos = getNextPosition();
        //vzdialenost k dalsej pozicii na cistenie
        RelativePosition b2np = new RelativePosition(getBotPosition(), nPos);
        //vzdialenosti medzi robotom a hracmi (v nasom pripade len 1 hrac)
        int[] sensorData = RelativePosition.readProximitySensor(getPlayerPositions(), getBotPosition());

        int command = 1;
        if (nPos.x == 0 && nPos.y == 0 && nPos.z == 0) {
            command = 3;
        }

        String verbose2 = "[";
        for (int val : sensorData) {
            verbose2 += val + " ";
        }
        verbose2 += "]";

        int distance = b2np.getDstInCm();
        if (distance <= 5) {
            distance = 0;
        }

        System.out.println("CleanBot.Update>>  Controller call: command=" + command + " angle=" + b2np.getCompassAngleInMinutes() + "\" distance=" + distance + "(" + b2np.getDstInCm() + ")cm sensorData=" + verbose2 + "");

        // get start time
        long startTime = System.nanoTime();
                
        // volanie riadiaceho programu
        Object[] answer = safetyController.updateAndEvaluate(command,
                b2np.getCompassAngleInMinutes(),
                distance,
                sensorData[0],
                sensorData[1],
                sensorData[2],
                sensorData[3],
                sensorData[4],
                sensorData[5],
                sensorData[6],
                sensorData[7]);
        
         // get end time, sub, write to file
        long endTime = System.nanoTime();
        long diff = endTime - startTime;
        (new Thread(new LoggerThread(diff))).start();


        //spracovanie odpovede riadiaceho programu
        Boolean botOn = (Boolean) answer[0];
        Boolean botCleaning = (Boolean) answer[1];
        Integer angleInMin = (Integer) answer[2];
        Integer dst = (Integer) answer[3];

        System.out.println("CleanBot.Update>>  Controller answer: on=" + botOn + " cleaning=" + botCleaning + " angle=" + angleInMin + "\" distance=" + dst + "cm");


        b2np.resetAngleInMinDstInCm(angleInMin, dst);

        Vector3f botDirection = new Vector3f(b2np.getVector().x, 0, b2np.getVector().y);
        Vector3f walkDirection = botDirection.normalize().multLocal(5f);

        if (dst < 5) {
            walkDirection.set(0, 0, 0);
        }
        go(walkDirection);

        if (botCleaning) {//zap./vyp. cistenia podla rozhodnutia riad. programu
            turnLightOn();
            clean();
            //check and kill
            List<Node> allPlayers = getAllPlayerPositions();
            for (Spatial spatial : allPlayers) {
                Vector3f spatialPosition = ((AbstractCharacter) spatial).getNodeForMovement().getLocalTranslation();
                float distanceToPlayer = spatialPosition.distance(getBotPosition());
                if (distanceToPlayer < 15) {
                    ((AbstractCharacter) spatial).setIsAlive(false);
                }
            }
        } else {
            turnLightOff();
        }

    }
}
