package models;

import app.Measure;
import com.jme3.light.Light;
import com.jme3.light.SpotLight;
import com.jme3.math.ColorRGBA;
import com.jme3.math.FastMath;
import com.jme3.math.Vector3f;
import com.jme3.scene.Node;
import java.util.ArrayList;
import java.util.Iterator;

public class DirtyAreaLocations {

    private ArrayList dirtyArea = new ArrayList<DirtyArea>();
    private Vector3f[] position = new Vector3f[]{new Vector3f(-25, 1, -13f), new Vector3f(-13, 1, 35f), new Vector3f(26, 1, 36f), new Vector3f(52, 1, -15f), new Vector3f(0, 1, -49f), new Vector3f(-36, 1, -31f)};
    private int initDirtiness = 100;
    private Node rootNode;

    public DirtyAreaLocations(Node rootNode) {
        this.rootNode = rootNode;
        fillStation();
    }

    public void fillStation() {
        for (int i = 0; i < position.length; i++) {
            SpotLight light = new SpotLight();
            light.setPosition(position[i].add(0, 70f, 0));
            light.setColor(ColorRGBA.Red);
            rootNode.addLight(light);

            getDirtyAreas().add(new DirtyArea(position[i], initDirtiness, light));
        }
    }

    public DirtyArea getCookie() {
        Iterator iterator = getDirtyAreas().iterator();
        DirtyArea dirtyAreaLocal;

        while (iterator.hasNext()) {
            dirtyAreaLocal = (DirtyArea) iterator.next();
            if (dirtyAreaLocal.dirtiness > 0) {
                return dirtyAreaLocal;
            }
        }
        return (DirtyArea) getDirtyAreas().get(getDirtyAreas().size() - 1);
    }

    public ArrayList getDirtyAreas() {
        return dirtyArea;
    }

    public class DirtyArea {

        private Vector3f position;
        private int dirtiness;
        private SpotLight light;

        public DirtyArea(Vector3f position, int dirtiness, SpotLight light) {
            this.position = position;
            this.dirtiness = dirtiness;
            this.light = light;
        }

        public void cleanOneUnit() {
            Vector3f lightPosition = light.getPosition();
            dirtiness--;
            if (dirtiness > 10) {
                light.setSpotOuterAngle(light.getSpotOuterAngle() - light.getSpotOuterAngle() / dirtiness);
                light.setSpotInnerAngle(light.getSpotOuterAngle() * 0.5f);
            } else {
                rootNode.removeLight(light);
            }
        }

        public int getDirtiness() {
            return this.dirtiness;
        }

        public Vector3f getPosition() {
            return position;
        }

        public SpotLight getLight() {
            return light;
        }
    }
}
